//
// avacalc.cpp
//
#include <iostream.h>
#include <objbase.h>
#include <stack> 
#include <string.h>
#include <stdlib.h>
#include "ifaccalc.h"      // Interface declarations
#include "Registry.h"   // Registry helper functions


///////////////////////////////////////////////////////////
//
// Global variables
//
static HMODULE g_hModule = NULL ;   // DLL module handle
static long g_cComponents = 0 ;     // Count of active components
static long g_cServerLocks = 0 ;    // Count of locks

// Friendly name of components
const char g_szFriendlyNameCalc[] = "AvaCalc, Calculator" ;
const char g_szFriendlyNameLcd[] = "AvaCalc, Lcd" ;
const char g_szFriendlyNameKernel[] = "AvaCalc, Kernel" ;

// Version-independent ProgID
const char g_szVerIndProgIDCalc[] = "Avalon.Calc" ;
const char g_szVerIndProgIDLcd[] = "Avalon.Lcd" ;
const char g_szVerIndProgIDKernel[] = "Avalon.Kernel" ;

// ProgID
const char g_szProgIDCalc[] = "Avalon.Calc.1" ;
const char g_szProgIDLcd[] = "Avalon.Lcd.1" ;
const char g_szProgIDKernel[] = "Avalon.Kernel.1" ;

//stack
using namespace std ;
typedef stack<double> STACK_DOUBLE;

//#define _MY_DEBUG_
//#define _MY_DEBUG

///////////////////////////////////////////////////////////
//   8  16    10 
int One8Digit(char cDigit)
{
	if(cDigit>='0' && cDigit<='8')
	{
		return cDigit-'0';
	}else
		return -1;

};
int One10Digit(char cDigit)
{
	if(cDigit>='0' && cDigit<='9')
	{
		return cDigit-'0';
	}else
		return -1;

};
int One16Digit(char cDigit)
{
	switch (cDigit)
	{
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
			return cDigit-'0';
			break;
	case 'a':
	case 'A':
			return 10;
			break;
	case 'b':
	case 'B':
			return 11;
			break;
	case 'c':
	case 'C':
			return 12;
			break;
	case 'd':
	case 'D':
			return 13;
			break;
	case 'e':
	case 'E':
			return 14;
			break;
	case 'f':
	case 'F':
			return 15;
			break;
	default:
			return -1;
			break;
	}
	return -1;

};

int OneDigit(char cDigit, AvaLcdMode lcdMode)
{
if(lcdMode==lcd_hex)
{
	return One16Digit(cDigit);
}
else if(lcdMode==lcd_dec)
{
	return One10Digit(cDigit);
}else if(lcdMode==lcd_oct)
{
	return One8Digit(cDigit);
}else return -1;
};

double StrToDouble(char *str, AvaLcdMode lcdMode)
{
double dRes=0;
int iDigit;
//cout << "Hello from StrToDouble('"<<str<<"')";
while(*str)
{
	iDigit=OneDigit(*str, lcdMode);
	if(iDigit==-1)
	{
		return -1;
		break;
	}

	if(lcdMode==lcd_hex)
	{
		dRes=dRes*16+iDigit;
	}
	else if(lcdMode==lcd_oct)
	{
		dRes=dRes*8+iDigit;
	}
	str++;
}

return dRes;
}
///////////////////////////////////////////////////////////
//Component 
//     .
//
class CAvaCalc:	public ICalcEngineStack,
				public ICalcEngineArif
{
public:
	// IUnknown
	virtual HRESULT __stdcall QueryInterface(const IID& iid, void** ppv) ;
	virtual ULONG __stdcall AddRef() ;
	virtual ULONG __stdcall Release() ;

	// Interface ICalcEngineStack
	//Reset -    
	virtual HRESULT __stdcall Reset();
	//ClearValue -    
	virtual HRESULT __stdcall PopValue(double& dValue);
	//PushValue -     
	virtual HRESULT __stdcall PushValue(const double dValue);
	//GetValue -     
	virtual HRESULT __stdcall GetValue(double& dValue);
	//ChangeValue -    
	virtual HRESULT __stdcall ChangeValue(const double dValue);
	//CountStack -     
	virtual HRESULT __stdcall CountStack(long& clCount);

	// Interface ICalcEngineArif
	virtual HRESULT __stdcall ArifAdd(double& dValue);
	virtual HRESULT __stdcall ArifSub(double& dValue);
	virtual HRESULT __stdcall ArifMul(double& dValue);
	virtual HRESULT __stdcall ArifDiv(double& dValue);

	// Constructor
	CAvaCalc() ;

	// Destructor
	~CAvaCalc() ;

private:
	// Reference count
	long m_cRef ;
	//
	STACK_DOUBLE dstackStack;
} ;


//
// Constructor
//
CAvaCalc::CAvaCalc() : m_cRef(1)
{ 
	InterlockedIncrement(&g_cComponents) ; 
}

//
// Destructor
//
CAvaCalc::~CAvaCalc() 
{ 
	InterlockedDecrement(&g_cComponents) ; 
}

//
// IUnknown implementation
//
HRESULT __stdcall CAvaCalc::QueryInterface(const IID& iid, void** ppv)
{    
	if (iid == IID_IUnknown)
	{
		*ppv = static_cast<ICalcEngineStack *>(this) ; 
	}
	else if (iid == IID_CALCSTACK)
	{
		*ppv = static_cast<ICalcEngineStack *>(this) ;
	}
	else if (iid == IID_CALCARIF)
	{
		*ppv = static_cast<ICalcEngineArif*>(this) ; 
	}
	else
	{
		*ppv = NULL ;
		return E_NOINTERFACE ;
	}
	reinterpret_cast<IUnknown*>(*ppv)->AddRef() ;
	return S_OK ;
}

ULONG __stdcall CAvaCalc::AddRef()
{
	return InterlockedIncrement(&m_cRef) ;
}

ULONG __stdcall CAvaCalc::Release() 
{
	if (InterlockedDecrement(&m_cRef) == 0)
	{
		delete this ;
		return 0 ;
	}
	return m_cRef ;
}


//Reset -    
HRESULT __stdcall CAvaCalc::Reset()
{

return E_NOTIMPL;
};

//ClearValue -    
HRESULT __stdcall CAvaCalc::PopValue(double& dValue)
{
	if(!dstackStack.empty())
	{

		dValue=dstackStack.top();
		dstackStack.pop();
#ifdef _MY_DEBUG_
long lCount;
lCount=dstackStack.size();
	cout<<"Pop("<<dValue<<"), size="<<lCount<<endl;
#endif // 
		return S_OK;
	} else
	{
		return E_AVACALC_NO_DATA;
	}
};

//PushValue -     
HRESULT __stdcall CAvaCalc::PushValue(const double dValue)
{

	dstackStack.push(dValue);
#ifdef _MY_DEBUG_
long lCount;
lCount=dstackStack.size();
	cout<<"Push("<<dValue<<"), size="<<lCount<<endl;
#endif // 
return S_OK;
};

//GetValue -     
HRESULT __stdcall CAvaCalc::GetValue(double& dValue)
{
	if(!dstackStack.empty())
	{
		dValue=dstackStack.top();
		return S_OK;
	} else
	{
		return E_AVACALC_NO_DATA;
	}
};


//ChangeValue -    
HRESULT __stdcall CAvaCalc::ChangeValue(const double dValue)
{
	if(!dstackStack.empty())
	{
		dstackStack.top()=dValue;
		return S_OK;
	} else
	{
		return E_AVACALC_NO_DATA;
	}
};


//CountStack -     
HRESULT __stdcall CAvaCalc::CountStack(long& clCount)
{
clCount=dstackStack.size();
return S_OK;
};


// Interface ICalcEngineArif
HRESULT __stdcall CAvaCalc::ArifAdd(double& dValue)
{
	HRESULT hr;
	long cStack;
	hr = CountStack(cStack);
	if(SUCCEEDED(hr))
	{ //   
		if(cStack>=2)
		{
			// 
			double par1, par2, res;
			PopValue(par1);
			PopValue(par2);
			res=par2+par1;
#ifdef _MY_DEBUG_
cout<<"Add:("<<par2<<"+"<<par1<<")="<<res<<endl;
#endif // 

			PushValue(res);
			dValue=res;
			return S_OK;
		} else //  
		{
			return E_AVACALC_NO_DATA;
		}
		return E_UNEXPECTED;
	}
return E_UNEXPECTED;
};

HRESULT __stdcall CAvaCalc::ArifSub(double& dValue)
{
	HRESULT hr;
	long cStack;
	hr = CountStack(cStack);
	if(SUCCEEDED(hr))
	{ //   
		if(cStack>=2)
		{
			// 
			double par1, par2, res;
			PopValue(par1);
			PopValue(par2);
			res=par2-par1;
#ifdef _MY_DEBUG_
cout<<"Sub:("<<par2<<"-"<<par1<<")="<<res<<endl;
#endif // 

			PushValue(res);
			dValue=res;
			return S_OK;
		} else //  
		{
			return E_AVACALC_NO_DATA;
		}
		return E_UNEXPECTED;
	}
return E_UNEXPECTED;
};

HRESULT __stdcall CAvaCalc::ArifMul(double& dValue)
{
	HRESULT hr;
	long cStack;
	hr = CountStack(cStack);
	if(SUCCEEDED(hr))
	{ //   
		if(cStack>=2)
		{
			// 
			double par1, par2, res;
			PopValue(par1);
			PopValue(par2);
			res=par2*par1;
#ifdef _MY_DEBUG_
cout<<"Mul:("<<par2<<"*"<<par1<<")="<<res<<endl;
#endif // 

			PushValue(res);
			dValue=res;
			return S_OK;
		} else //  
		{
			return E_AVACALC_NO_DATA;
		}
		return E_UNEXPECTED;
	}
return E_UNEXPECTED;
};

HRESULT __stdcall CAvaCalc::ArifDiv(double& dValue)
{
	HRESULT hr;
	long cStack;
	hr = CountStack(cStack);
	if(SUCCEEDED(hr))
	{ //   
		if(cStack>=2)
		{
			// 
			double par1, par2, res;
			PopValue(par1);
			PopValue(par2);
			res=par2/par1;
#ifdef _MY_DEBUG_
cout<<"Div:("<<par2<<"/"<<par1<<")="<<res<<endl;
#endif // 
			PushValue(res);
			dValue=res;
			return S_OK;
		} else //  
		{
			return E_AVACALC_NO_DATA;
		}
		return E_UNEXPECTED;
	}
return E_UNEXPECTED;
};

///////////////////////////////////////////////////////////
//Component 
//      
//   

class CAvaLcd : public ILcdAccess,
				public ILcdEdit,
				public ILcdMode
{
public:
	//  
	CAvaLcd();
	~CAvaLcd();

	
	// IUnknown
	virtual HRESULT __stdcall QueryInterface(const IID& iid, void** ppv) ;
	virtual ULONG __stdcall AddRef() ;
	virtual ULONG __stdcall Release() ;

	//ILcdAccess,
	virtual HRESULT __stdcall GetDouble(double& dValue);
	virtual HRESULT __stdcall SetDouble(const double dValue);

	//    
	//      
	//   : bin, oct, dec, hex
	virtual HRESULT __stdcall GetString(char* sz);
	//  ,    - 
	virtual HRESULT __stdcall SetString(char* sz);

	//ILcdEdit
	virtual HRESULT __stdcall AddLast(double dDigit);
	virtual HRESULT __stdcall DelLast();
	virtual HRESULT __stdcall AddLast(char* szDigit);

	//ILcdMode
	// ,   
	virtual HRESULT __stdcall SetMode(const AvaLcdMode mode);
	virtual HRESULT __stdcall GetMode(AvaLcdMode* mode);
	
	//SetBinMode SetOctMode SetDecMode SetHexNode - 
	// SetMode
	virtual HRESULT __stdcall SetBinMode();
	virtual HRESULT __stdcall SetOctMode();
	virtual HRESULT __stdcall SetDecMode();
	virtual HRESULT __stdcall SetHexMode();

	//      
	virtual HRESULT __stdcall SetBinLen(const long lLen);
	virtual HRESULT __stdcall GetBinLen(long& lLen);
	virtual HRESULT __stdcall SetOctLen(const long lLen);
	virtual HRESULT __stdcall GetOctLen(long& lLen);
	virtual HRESULT __stdcall SetHexLen(const long lLen);
	virtual HRESULT __stdcall GetHexLen(long& lLen);
	virtual HRESULT __stdcall SetDecBeforePoint(const long lLen);
	virtual HRESULT __stdcall GetDecBeforePoint(long& lLen);
	virtual HRESULT __stdcall SetAfterPoint(const long lLen);
	virtual HRESULT __stdcall GetAfterPoint(long& lLen);

private:
	// Reference count
	long m_cRef ;
	AvaLcdMode lcdMode;
	double dVal;
	int iOctLen, iHexLen, iDecBefore, iDecAfter;
};

// CAvaLcd
CAvaLcd::CAvaLcd() : m_cRef(1)
{ 
	//      
	InterlockedIncrement(&g_cComponents) ; 

	//    
	// = 0
	dVal=0;
	// - 
	lcdMode = lcd_dec;
	//  (   
	iOctLen=10;
	iHexLen=10;
	iDecBefore=10;
	iDecAfter=3;
}

CAvaLcd::~CAvaLcd() 
{
	//      
	InterlockedDecrement(&g_cComponents) ; 
}


//
// IUnknown implementation
//
HRESULT __stdcall CAvaLcd::QueryInterface(const IID& iid, void** ppv)
{    
	if (iid == IID_IUnknown)
	{
		*ppv = static_cast<ILcdAccess *>(this) ; 
	}
	else if (iid == IID_LCDACCESS)
	{
		*ppv = static_cast<ILcdAccess *>(this) ;
	}
	else if (iid == IID_LCDMODE)
	{
		*ppv = static_cast<ILcdMode *>(this) ; 
	}
	else if (iid == IID_LCDEDIT)
	{
		*ppv = static_cast<ILcdEdit *>(this) ; 
	}
	else
	{
		*ppv = NULL ;
		return E_NOINTERFACE ;
	}
	reinterpret_cast<IUnknown*>(*ppv)->AddRef() ;
	return S_OK ;
}

ULONG __stdcall CAvaLcd::AddRef()
{
	return InterlockedIncrement(&m_cRef) ;
}

ULONG __stdcall CAvaLcd::Release() 
{
	if (InterlockedDecrement(&m_cRef) == 0)
	{
		delete this ;
		return 0 ;
	}
	return m_cRef ;
}

//ILcdAccess,
HRESULT __stdcall CAvaLcd::GetDouble(double& dValue)
{
	dValue=dVal;
	return S_OK;
};
HRESULT __stdcall CAvaLcd::SetDouble(const double dValue)
{
	dVal=dValue;
	return S_OK;
};

	//    
	//      
	//   : bin, oct, dec, hex
HRESULT __stdcall CAvaLcd::GetString(char* sz)
{
	//itoa
	
	switch (lcdMode)
	{
	case lcd_oct:
		sprintf(sz, "O%o", int(dVal));
		break;
	case lcd_dec:
		sprintf(sz, "%f", dVal);
		break;
	case lcd_hex:
		sprintf(sz, "Ox%X", int(dVal));
		break;
	default:
		sprintf(sz, "%f", dVal);
		break;
	};
	return S_OK;
};
	//  ,    - 
HRESULT __stdcall CAvaLcd::SetString(char* sz)
{
	double dTemp;

	switch (lcdMode)
	{
	case lcd_oct:
	case lcd_hex:
				//cout<<"Hello from SetString, sz="<<sz;
				dTemp=StrToDouble(sz, lcdMode);
				//cout<<" double="<<dTemp;
				if(dTemp!=-1)
					dVal=dTemp;
				break;
	case lcd_dec:
				dVal=atof(sz);
	default:
		break;
	};
	return S_OK;
};

	//ILcdEdit
HRESULT __stdcall CAvaLcd::AddLast(double dDigit)
{
//cout<<"In addLast, with dig="<<dDigit<<endl;	
//int iLen;
//char* szBuf=new char(40);
int iAf, i=1;

	switch (lcdMode)
	{
	case lcd_oct:
		dVal=dVal*8+int(dDigit);
		break;
	case lcd_dec:
		if(dVal==int(dVal))//  
		{
			dVal=dVal*10+dDigit;
		}else
		{
//cout<<"iDecAfter"<<iDecAfter<<endl;
			for(iAf=0; iAf<iDecAfter; iAf++)
			{//     
//cout<<"iAf="<<iAf<<", i="<<i<<endl;
				if((dVal*i)==int(dVal*i))
				{//    
					dVal=(dVal*i*10+dDigit)/(i*10);
					break;
//cout<<"level after point="<<iAf<<endl;
				}
				i*=10;

			}
//cout<<"that`s all points"<<endl;
		};

//cout<<"In addLast, GetString="<<szBuf<<" and len="<<iLen<<endl;	
		break;
	case lcd_hex:
		dVal=dVal*16+int(dDigit);
		break;
	default:
		break;
	};	
	return S_OK;
};

HRESULT __stdcall CAvaLcd::AddLast(char* szDigit)
{
//cout<<"In addLast, with str="<<szDigit<<endl;	
	AddLast(double (OneDigit(*szDigit, lcdMode)));
	return S_OK;
};

HRESULT __stdcall CAvaLcd::DelLast()
{
//char *buf=new char(40);
int iAf, i;
	switch (lcdMode)
	{
	case lcd_oct:
		dVal=int(dVal/8);
		break;
	case lcd_dec:
		if(dVal==int(dVal))//  
		{
			dVal=dVal/10;
		}else
		{
//cout<<"iDecAfter"<<iDecAfter<<endl;
			i=10^3;
			for(iAf=iDecAfter; iAf>0; iAf++)
			{//     
//cout<<"iAf="<<iAf<<", i="<<i<<endl;
				if((dVal*i)==int(dVal*i))
				{//    
					dVal=int((dVal*i))/(i*10);
					break;
//cout<<"level after point="<<iAf<<endl;
				}
				i*=10;

			}
//cout<<"that`s all points"<<endl;
		};
		break;
	case lcd_hex:
		dVal=int(dVal/16);
		break;
	default:
		break;
	};	
//delete buf;
	return S_OK;
};

	//ILcdMode
	// ,   
HRESULT __stdcall CAvaLcd::SetMode(AvaLcdMode mode)
{
	lcdMode=mode;
	return S_OK;
};
HRESULT __stdcall CAvaLcd::GetMode(AvaLcdMode* mode)
{
	*mode=lcdMode;
	return S_OK;
};
	
	//SetBinMode SetOctMode SetDecMode SetHexNode - 
	// SetMode
HRESULT __stdcall CAvaLcd::SetBinMode()
{
	return E_NOTIMPL;
};
HRESULT __stdcall CAvaLcd::SetOctMode()
{
	SetMode(lcd_oct);
	return S_OK;
};
HRESULT __stdcall CAvaLcd::SetDecMode()
{
	SetMode(lcd_dec);
	return S_OK;
};
HRESULT __stdcall CAvaLcd::SetHexMode()
{
	SetMode(lcd_hex);
	return S_OK;
};

	//      
HRESULT __stdcall CAvaLcd::SetBinLen(const long lLen)
{
	
	return E_NOTIMPL;
};
HRESULT __stdcall CAvaLcd::GetBinLen(long& lLen)
{
	
	return E_NOTIMPL;
};
HRESULT __stdcall CAvaLcd::SetOctLen(const long lLen)
{
	iOctLen=lLen;
	return S_OK;
};
HRESULT __stdcall CAvaLcd::GetOctLen(long& lLen)
{
	lLen=iOctLen;
	return S_OK;
};
HRESULT __stdcall CAvaLcd::SetHexLen(const long lLen)
{
	iHexLen=10;
	return S_OK;
};
HRESULT __stdcall CAvaLcd::GetHexLen(long& lLen)
{
	lLen=iHexLen;
	return S_OK;
};
HRESULT __stdcall CAvaLcd::SetDecBeforePoint(const long lLen)
{
	iDecBefore=lLen;
	return S_OK;
};
HRESULT __stdcall CAvaLcd::GetDecBeforePoint(long& lLen)
{
	lLen=iDecBefore;
	return S_OK;
};
HRESULT __stdcall CAvaLcd::SetAfterPoint(const long lLen)
{
	iDecAfter=lLen;
	return S_OK;
};
HRESULT __stdcall CAvaLcd::GetAfterPoint(long& lLen)
{
	lLen=iDecAfter;
	return S_OK;
};

///////////////////////////////////////////////////////////
//Component 
//      
//

class CAvaKernel :	public ICalcKernelProcess,
					public ILcdAccess
{
public:
	//  
	CAvaKernel();
	~CAvaKernel();

	
	// IUnknown
	virtual HRESULT __stdcall QueryInterface(const IID& iid, void** ppv) ;
	virtual ULONG __stdcall AddRef() ;
	virtual ULONG __stdcall Release() ;

	//ILcdAccess,
	virtual HRESULT __stdcall GetDouble(double& dValue);
	virtual HRESULT __stdcall SetDouble(const double dValue);

	//    
	//      
	//   : bin, oct, dec, hex
	virtual HRESULT __stdcall GetString(char* sz);
	//  ,    - 
	virtual HRESULT __stdcall SetString(char* sz);

	//CAvaKernel
	virtual HRESULT __stdcall NextCommand(char* sz);


	//  ,   
	BOOL isEditCmd(char* szCmd);
	BOOL isOperatingCmd(char* szCmd);
	BOOL isEditing();
	BOOL isOperating();
	void SetEditing();
	void SetOperating();
private:
	// Reference count
	long m_cRef ;
	AvaKernelMode kernelMode;

	//   
	ICalcEngineStack*	pStack;
	ICalcEngineArif*	pArif;
	ILcdAccess*			pAccess;
	ILcdEdit*			pEdit;
	ILcdMode*			pMode;
};

//
// CAvaLcd
CAvaKernel::CAvaKernel() : m_cRef(1)
{ 

	//      
	InterlockedIncrement(&g_cComponents) ; 


	//       .
	HRESULT hr;
	//    
	// 
	hr = ::CoCreateInstance(CLSID_AVACALCENGINE,
				NULL,
				CLSCTX_INPROC_SERVER,
				IID_CALCSTACK,
				(void**)&pStack);

	// 
	hr = pStack->QueryInterface(IID_CALCARIF, (void**)&pArif);

	//    
	// 
	hr = ::CoCreateInstance(CLSID_AVACALCLCD, 
				NULL,
				CLSCTX_INPROC_SERVER,
				IID_LCDACCESS,
				(void**)&pAccess);

	//  
	hr = pAccess->QueryInterface(IID_LCDMODE, (void**)&pMode);

	// 
	hr = pAccess->QueryInterface(IID_LCDEDIT, (void**)&pEdit);

	//
	SetEditing();
	pMode->SetDecMode();
	pAccess->SetDouble(0.0);
//    ,
//  ,      ,    
//,     .
//     

}

CAvaKernel::~CAvaKernel() 
{
	//      
	InterlockedDecrement(&g_cComponents) ; 
	
	// 
	pStack->Release();
	pArif->Release();
	pAccess->Release();
	pEdit->Release();
	pMode->Release();
}


//
// IUnknown implementation
//
HRESULT __stdcall CAvaKernel::QueryInterface(const IID& iid, void** ppv)
{    
	if (iid == IID_IUnknown)
	{
		*ppv = static_cast<ILcdAccess *>(this) ; 
	}
	else if (iid == IID_LCDACCESS)
	{
		*ppv = static_cast<ILcdAccess *>(this) ;
	}
	else if (iid == IID_CALCKERNELPROC)
	{
		*ppv = static_cast<CAvaKernel *>(this) ; 
	}
	else
	{
		*ppv = NULL ;
		return E_NOINTERFACE ;
	}
	reinterpret_cast<IUnknown*>(*ppv)->AddRef() ;
	return S_OK ;
}

ULONG __stdcall CAvaKernel::AddRef()
{
	return InterlockedIncrement(&m_cRef) ;
}

ULONG __stdcall CAvaKernel::Release() 
{
	if (InterlockedDecrement(&m_cRef) == 0)
	{
		delete this ;
		return 0 ;
	}
	return m_cRef ;
}

//ILcdAccess,
HRESULT __stdcall CAvaKernel::GetDouble(double& dValue)
{
	pAccess->GetDouble(dValue);
	return S_OK;
};
HRESULT __stdcall CAvaKernel::SetDouble(const double dValue)
{
	pAccess->SetDouble(dValue);
	return S_OK;
};

	//    
	//      
	//   : bin, oct, dec, hex
HRESULT __stdcall CAvaKernel::GetString(char* sz)
{
	pAccess->GetString(sz);
	return S_OK;
};
	//  ,    - 
HRESULT __stdcall CAvaKernel::SetString(char* sz)
{
	pAccess->SetString(sz);
	return S_OK;
};

//       
//
HRESULT __stdcall CAvaKernel::NextCommand(char* sz)
{
#ifdef _MY_DEBUG
cout<<"HAVE A COMMAND:"<<sz<<endl;
#endif // 
	if(isEditCmd(sz))
	{//  (   
#ifdef _MY_DEBUG
cout<<"isEditCmd"<<endl;
#endif // 
		if(isOperating())
		{//      ,   
		 // Lcd  0
#ifdef _MY_DEBUG
cout<<"isOperating()"<<endl;
#endif // 

			pAccess->SetDouble(0.0);
			//   
			SetEditing();
		};
#ifdef _MY_DEBUG
cout<<". STAY edit"<<endl;
#endif // 
		//   
		if(strcmp(sz,"\b")==0) //,   
		{
#ifdef _MY_DEBUG
cout<<"bspace"<<endl;
#endif // 

			pEdit->DelLast();
		}else
		{//
#ifdef _MY_DEBUG
cout<<"have a digit '"<<sz<<"'"<<endl;
#endif // 
			pEdit->AddLast(sz);
		};

	}
	if(isOperatingCmd(sz))
	{
#ifdef _MY_DEBUG
cout<<"oper cmd"<<endl;
#endif // 
		if(isEditing())
		{
#ifdef _MY_DEBUG
cout<<", WAS editing"<<endl;
#endif // 
			//    ,  
			//     
			double dToStack;
			pAccess->GetDouble(dToStack);
//			pStack->PushValue(dToStack);
			SetOperating();
		};
#ifdef _MY_DEBUG
cout<<". STAY operating"<<endl;
#endif // 

		//   
		//
		double dArif;
			if(strcmp(sz, "Clear")==0)
			{
				//   0  
				//   
				pAccess->SetDouble(0.0);
				pStack->ChangeValue(0.0);
#ifdef _MY_DEBUG
cout<<"Clear"<<endl;
#endif // 
			}
			if(strcmp(sz,"Push")==0)
			{
				//     
				double dToStack;
				pAccess->GetDouble(dToStack);
				pStack->PushValue(dToStack);
#ifdef _MY_DEBUG
cout<<"Push"<<endl;
#endif // 
			}
			if(strcmp(sz,"Add")==0)
			{
				pArif->ArifAdd(dArif);
				pAccess->SetDouble(dArif);
//				pStack->PushValue(dArif);
#ifdef _MY_DEBUG
cout<<"Add"<<endl;
#endif // 
			}
			if(strcmp(sz,"Sub")==0)
			{
				pArif->ArifSub(dArif);
				pAccess->SetDouble(dArif);
//				pStack->PushValue(dArif);
#ifdef _MY_DEBUG
cout<<"Sub"<<endl;
#endif // 
			}
			if(strcmp(sz,"Mul")==0)
			{
				pArif->ArifMul(dArif);
				pAccess->SetDouble(dArif);
//				pStack->PushValue(dArif);
#ifdef _MY_DEBUG
cout<<"Mul"<<endl;
#endif // 
			}
			if(strcmp(sz,"Div")==0)
			{
				pArif->ArifDiv(dArif);
				pAccess->SetDouble(dArif);

#ifdef _MY_DEBUG
cout<<"Div"<<endl;
#endif // 
			}
			if(strcmp(sz,"Oct")==0)
			{
				pMode->SetOctMode();
#ifdef _MY_DEBUG
cout<<"Oct"<<endl;
#endif // 
			}
			if(strcmp(sz,"Dec")==0)
			{
				pMode->SetDecMode();
#ifdef _MY_DEBUG
cout<<"Dec"<<endl;
#endif // 
			}
			if(strcmp(sz,"Hex")==0)
			{
				pMode->SetHexMode();
#ifdef _MY_DEBUG
cout<<"Hex"<<endl;
#endif // 
			}
			if(strcmp(sz,"Bin")==0)
			{
				pMode->SetBinMode();
#ifdef _MY_DEBUG
cout<<"Bin"<<endl;
#endif // 
			}
	};

return S_OK;
}

// ,    .
//     ?
BOOL CAvaKernel::isEditCmd(char* szCmd)
{

	if(strcmp(szCmd, "\b")==0 ||
   strcmp(szCmd, "0" )==0 ||
	strcmp(szCmd, "1" )==0 ||
	strcmp(szCmd, "2" )==0 ||
	strcmp(szCmd, "3" )==0 ||
	strcmp(szCmd, "4" )==0 ||
	strcmp(szCmd, "5" )==0 ||
	strcmp(szCmd, "6" )==0 ||
	strcmp(szCmd, "7" )==0 ||
	strcmp(szCmd, "8" )==0 ||
	strcmp(szCmd, "9" )==0 ||
	strcmp(szCmd, "A" )==0 ||
	strcmp(szCmd, "a" )==0 ||
	strcmp(szCmd, "B" )==0 ||
	strcmp(szCmd, "b" )==0 ||
	strcmp(szCmd, "C" )==0 ||
	strcmp(szCmd, "c" )==0 ||
	strcmp(szCmd, "D" )==0 ||
	strcmp(szCmd, "d" )==0 ||
	strcmp(szCmd, "E" )==0 ||
	strcmp(szCmd, "e" )==0 ||
	strcmp(szCmd, "F" )==0 ||
	strcmp(szCmd, "f" )==0 )
return true;
else		return false;
};


//   ?
BOOL CAvaKernel::isOperatingCmd(char* szCmd)
{
if(strcmp(szCmd,"Clear")==0 ||
	strcmp(szCmd, "Push")==0 ||
	strcmp(szCmd, "Add")==0 ||
	strcmp(szCmd, "Sub")==0 ||
	strcmp(szCmd, "Mul")==0 ||
	strcmp(szCmd, "Div")==0 ||
	strcmp(szCmd, "Oct")==0 ||
	strcmp(szCmd, "Dec")==0 ||
	strcmp(szCmd, "Hex")==0 ||
	strcmp(szCmd, "Bin")==0)
		return true;
else
		return false;
};


// ,    .
// 
BOOL CAvaKernel::isEditing()
{
	if(kernelMode == kernel_editing)
		return true;
	else
		return false;
};

BOOL CAvaKernel::isOperating()
{
	if(kernelMode == kernel_operating)
		return true;
	else
		return false;
};

void CAvaKernel::SetEditing()
{
	kernelMode = kernel_editing;
};

void CAvaKernel::SetOperating()
{
	kernelMode = kernel_operating;
};


///////////////////////////////////////////////////////////
//
// Class factory of CAvaCalc
//
class CAvaCalcFactory : public IClassFactory
{
public:
	// IUnknown
	virtual HRESULT __stdcall QueryInterface(const IID& iid, void** ppv) ;         
	virtual ULONG   __stdcall AddRef() ;
	virtual ULONG   __stdcall Release() ;

	// Interface IClassFactory
	virtual HRESULT __stdcall CreateInstance(IUnknown* pUnknownOuter,
	                                         const IID& iid,
	                                         void** ppv) ;
	virtual HRESULT __stdcall LockServer(BOOL bLock) ; 

	// Constructor
	CAvaCalcFactory() : m_cRef(1) {}

	// Destructor
	~CAvaCalcFactory() {}

private:
	long m_cRef ;
} ;

//
// Class factory IUnknown implementation
//
HRESULT __stdcall CAvaCalcFactory::QueryInterface(const IID& iid, void** ppv)
{    
	if ((iid == IID_IUnknown) || (iid == IID_IClassFactory))
	{
		*ppv = static_cast<IClassFactory*>(this) ; 
	}
	else
	{
		*ppv = NULL ;
		return E_NOINTERFACE ;
	}
	reinterpret_cast<IUnknown*>(*ppv)->AddRef() ;
	return S_OK ;
}

ULONG __stdcall CAvaCalcFactory::AddRef()
{
	return InterlockedIncrement(&m_cRef) ;
}

ULONG __stdcall CAvaCalcFactory::Release() 
{
	if (InterlockedDecrement(&m_cRef) == 0)
	{
		delete this ;
		return 0 ;
	}
	return m_cRef ;
}

//
// IClassFactory implementation
//
HRESULT __stdcall CAvaCalcFactory::CreateInstance(IUnknown* pUnknownOuter,
                                           const IID& iid,
                                           void** ppv) 
{
	// Cannot aggregate.
	if (pUnknownOuter != NULL)
	{
		return CLASS_E_NOAGGREGATION ;
	}

	// Create component.
	CAvaCalc* pAvaCalc = new CAvaCalc ;
	if (pAvaCalc == NULL)
	{
		return E_OUTOFMEMORY ;
	}

	// Get the requested interface.
	HRESULT hr = pAvaCalc->QueryInterface(iid, ppv) ;

	// Release the IUnknown pointer.
	// (If QueryInterface failed, component will delete itself.)
	pAvaCalc->Release() ;
	return hr ;
}

// LockServer
HRESULT __stdcall CAvaCalcFactory::LockServer(BOOL bLock) 
{
	if (bLock)
	{
		InterlockedIncrement(&g_cServerLocks) ; 
	}
	else
	{
		InterlockedDecrement(&g_cServerLocks) ;
	}
	return S_OK ;
}

///////////////////////////////////////////////////////////
//
// Class factory of CAvaLcd
//
class CAvaLcdFactory : public IClassFactory
{
public:
	// IUnknown
	virtual HRESULT __stdcall QueryInterface(const IID& iid, void** ppv) ;         
	virtual ULONG   __stdcall AddRef() ;
	virtual ULONG   __stdcall Release() ;

	// Interface IClassFactory
	virtual HRESULT __stdcall CreateInstance(IUnknown* pUnknownOuter,
	                                         const IID& iid,
	                                         void** ppv) ;
	virtual HRESULT __stdcall LockServer(BOOL bLock) ; 

	// Constructor
	CAvaLcdFactory() : m_cRef(1) {}

	// Destructor
	~CAvaLcdFactory() {}

private:
	long m_cRef ;
} ;

//
// Class factory IUnknown implementation
//
HRESULT __stdcall CAvaLcdFactory::QueryInterface(const IID& iid, void** ppv)
{    
	if ((iid == IID_IUnknown) || (iid == IID_IClassFactory))
	{
		*ppv = static_cast<IClassFactory*>(this) ; 
	}
	else
	{
		*ppv = NULL ;
		return E_NOINTERFACE ;
	}
	reinterpret_cast<IUnknown*>(*ppv)->AddRef() ;
	return S_OK ;
}

ULONG __stdcall CAvaLcdFactory::AddRef()
{
	return InterlockedIncrement(&m_cRef) ;
}

ULONG __stdcall CAvaLcdFactory::Release() 
{
	if (InterlockedDecrement(&m_cRef) == 0)
	{
		delete this ;
		return 0 ;
	}
	return m_cRef ;
}

//
// IClassFactory implementation
//
HRESULT __stdcall CAvaLcdFactory::CreateInstance(IUnknown* pUnknownOuter,
                                           const IID& iid,
                                           void** ppv) 
{
	// Cannot aggregate.
	if (pUnknownOuter != NULL)
	{
		return CLASS_E_NOAGGREGATION ;
	}

	// Create component.
	CAvaLcd* pAvaLcd = new CAvaLcd ;
	if (pAvaLcd == NULL)
	{
		return E_OUTOFMEMORY ;
	}

	// Get the requested interface.
	HRESULT hr = pAvaLcd->QueryInterface(iid, ppv) ;

	// Release the IUnknown pointer.
	// (If QueryInterface failed, component will delete itself.)
	pAvaLcd->Release() ;
	return hr ;
}

// LockServer
HRESULT __stdcall CAvaLcdFactory::LockServer(BOOL bLock) 
{
	if (bLock)
	{
		InterlockedIncrement(&g_cServerLocks) ; 
	}
	else
	{
		InterlockedDecrement(&g_cServerLocks) ;
	}
	return S_OK ;
}


///////////////////////////////////////////////////////////
//
// Class factory of CAvaKernel
//
class CAvaKernelFactory : public IClassFactory
{
public:
	// IUnknown
	virtual HRESULT __stdcall QueryInterface(const IID& iid, void** ppv) ;         
	virtual ULONG   __stdcall AddRef() ;
	virtual ULONG   __stdcall Release() ;

	// Interface IClassFactory
	virtual HRESULT __stdcall CreateInstance(IUnknown* pUnknownOuter,
	                                         const IID& iid,
	                                         void** ppv) ;
	virtual HRESULT __stdcall LockServer(BOOL bLock) ; 

	// Constructor
	CAvaKernelFactory() : m_cRef(1) {}

	// Destructor
	~CAvaKernelFactory() {}

private:
	long m_cRef ;
} ;

//
// Class factory IUnknown implementation
//
HRESULT __stdcall CAvaKernelFactory::QueryInterface(const IID& iid, void** ppv)
{    
	if ((iid == IID_IUnknown) || (iid == IID_IClassFactory))
	{
		*ppv = static_cast<IClassFactory*>(this) ; 
	}
	else
	{
		*ppv = NULL ;
		return E_NOINTERFACE ;
	}
	reinterpret_cast<IUnknown*>(*ppv)->AddRef() ;
	return S_OK ;
}

ULONG __stdcall CAvaKernelFactory::AddRef()
{
	return InterlockedIncrement(&m_cRef) ;
}

ULONG __stdcall CAvaKernelFactory::Release() 
{
	if (InterlockedDecrement(&m_cRef) == 0)
	{
		delete this ;
		return 0 ;
	}
	return m_cRef ;
}

//
// IClassFactory implementation
//
HRESULT __stdcall CAvaKernelFactory::CreateInstance(IUnknown* pUnknownOuter,
                                           const IID& iid,
                                           void** ppv) 
{
	// Cannot aggregate.
	if (pUnknownOuter != NULL)
	{
		return CLASS_E_NOAGGREGATION ;
	}
	//TARCE("In Factory::CreateInstance");
	// Create component.
	CAvaKernel* pAvaKernel = new CAvaKernel;

	
	if (pAvaKernel == NULL)
	{
		return E_OUTOFMEMORY ;
	}
	// Get the requested interface.
	HRESULT hr = pAvaKernel->QueryInterface(iid, ppv) ;
if(SUCCEEDED(hr))
{
//cout<<"pAvaKernel->QueryInterface(iid, ppv) failed!"<<endl;
}
	// Release the IUnknown pointer.
	// (If QueryInterface failed, component will delete itself.)
	pAvaKernel->Release() ;
//cout<<"pAvaKernel->Release()"<<endl;

	return hr ;
}

// LockServer
HRESULT __stdcall CAvaKernelFactory::LockServer(BOOL bLock) 
{
	if (bLock)
	{
		InterlockedIncrement(&g_cServerLocks) ; 
	}
	else
	{
		InterlockedDecrement(&g_cServerLocks) ;
	}
	return S_OK ;
}

///////////////////////////////////////////////////////////
//
// Exported functions
//

//
// Can DLL unload now?
//
STDAPI DllCanUnloadNow()
{
	if ((g_cComponents == 0) && (g_cServerLocks == 0))
	{
		return S_OK ;
	}
	else
	{
		return S_FALSE ;
	}
}

//
// Get class factory
//
STDAPI DllGetClassObject(const CLSID& clsid,
                         const IID& iid,
                         void** ppv)
{
	HRESULT hr;
	// Can we create this component?
	if (clsid == CLSID_AVACALCKERNEL)
	{
	// Create class factory.
	CAvaKernelFactory* pAvaKernelFactory = new CAvaKernelFactory ;  // No AddRef in constructor
	if (pAvaKernelFactory == NULL)
	{
		return E_OUTOFMEMORY ;
	}
	// Get requested interface.
	hr = pAvaKernelFactory->QueryInterface(iid, ppv) ;
	pAvaKernelFactory->Release() ;

	return hr ;
	}else 	if (clsid == CLSID_AVACALCENGINE)
	{
	// Create class factory.
	CAvaCalcFactory* pAvaCalcFactory = new CAvaCalcFactory ;  // No AddRef in constructor
	if (pAvaCalcFactory == NULL)
	{
		return E_OUTOFMEMORY ;
	}

	// Get requested interface.
	hr = pAvaCalcFactory->QueryInterface(iid, ppv) ;
	pAvaCalcFactory->Release() ;

	
	return hr ;
	}else  if(clsid==CLSID_AVACALCLCD)
	{
	// Create class factory.
	CAvaLcdFactory* pAvaLcdFactory = new CAvaLcdFactory ;  // No AddRef in constructor
	if (pAvaLcdFactory == NULL)
	{
		return E_OUTOFMEMORY ;
	}

	// Get requested interface.
	hr = pAvaLcdFactory->QueryInterface(iid, ppv) ;
	pAvaLcdFactory->Release() ;
	return hr ;
	} else
	{
		return CLASS_E_CLASSNOTAVAILABLE ;
	}
}

//CLSID_AVACALCENGINE, CLSID_AVACALCLCD, CLSID_AVACALCKERNEL
// Server registration
//
STDAPI DllRegisterServer()
{
HRESULT res1, res2, res3;
	res1 = RegisterServer(g_hModule, 
	                      CLSID_AVACALCENGINE,
	                      g_szFriendlyNameCalc,
	                      g_szVerIndProgIDCalc,
	                      g_szProgIDCalc) ;
	res2 = RegisterServer(g_hModule, 
	                      CLSID_AVACALCLCD,
	                      g_szFriendlyNameLcd,
	                      g_szVerIndProgIDLcd,
	                      g_szProgIDLcd) ;
	res3 = RegisterServer(g_hModule, 
	                      CLSID_AVACALCKERNEL,
	                      g_szFriendlyNameKernel,
	                      g_szVerIndProgIDKernel,
	                      g_szProgIDKernel) ;
if(SUCCEEDED(res1)&&SUCCEEDED(res2)&&SUCCEEDED(res3))
	{return S_OK;}
	else 
	{return E_FAIL;}
}


//
// Server unregistration
//
STDAPI DllUnregisterServer()
{
	HRESULT res1, res2, res3;
	res1 = UnregisterServer(CLSID_AVACALCENGINE,
	                        g_szVerIndProgIDCalc,
	                        g_szProgIDCalc) ;
	res2 = UnregisterServer(CLSID_AVACALCLCD,
	                        g_szVerIndProgIDLcd,
	                        g_szProgIDLcd) ;
	res3 = UnregisterServer(CLSID_AVACALCKERNEL,
	                        g_szVerIndProgIDKernel,
	                        g_szProgIDKernel) ;
	if(SUCCEEDED(res1)&&SUCCEEDED(res2)&&SUCCEEDED(res3))
		return S_OK;
	else 
		return E_FAIL;
}

///////////////////////////////////////////////////////////
//
// DLL module information
//
BOOL APIENTRY DllMain(HANDLE hModule,
                      DWORD dwReason,
                      void* lpReserved)
{
	if (dwReason == DLL_PROCESS_ATTACH)
	{
		g_hModule = (HMODULE)hModule ;
	}
	return TRUE ;
}
